<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 22fdfd9bca91 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2022-12-02 12:31 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_captcha_captcha{

	/**
	 * display the image
	 *
	 * @return string
	 */
	public static function display(we_captcha_image $image, string $type = 'gif'): string{
		[$type, $data] = self::get($image, $type);

		header('Expires: ' . gmdate('D, d M Y H:i:s') . ' GMT');
		header('Cache-Control: no-cache, must-revalidate, max-age=0');
		header('Pragma: no-cache');
		header('Content-type: ' . $type);
		return $data;
	}

	/**
	 *
	 * @param we_captcha_image $image
	 * @param string $type
	 * @return array{string,string}
	 */
	public static function get(we_captcha_image $image, string $type = 'gif'): array{
		$code = '';
		$im = $image->get($code);
		// save the code to the memory
		self::save($code);

		ob_start();
		switch($type){
			case 'jpg':
				$ctype = we_base_util::MIMETYPES[$type][0];
				imagejpeg($im);
				imagedestroy($im);
				break;
			case 'png':
				$ctype = we_base_util::MIMETYPES[$type][0];
				imagepng($im);
				imagedestroy($im);
				break;
			case 'webp':
				$ctype = we_base_util::MIMETYPES[$type][0];
				imagewebp($im);
				imagedestroy($im);
				break;
			case 'gif':
			default:
				$ctype = we_base_util::MIMETYPES['gif'][0];
				imagegif($im);
				imagedestroy($im);
				break;
		}
		return [$ctype, ob_get_clean()];
	}

	public static function check(string $captcha, string $type = 'captcha', bool $strict = false): bool{
		static $valid = [];
		if(isset($valid[$captcha])){
			return true;
		}
		$db = DB_WE::$inst;
		if(!we_cache_we::isDBCache() && !we_cache_we::isNoCache() && we_cache_we::clean('cap_' . $type . '_' . $captcha)){
			$valid[$captcha] = true;
			return true;
		}
//don't check IP, since IPv4/IPv6 switches between requests
		$db->query('DELETE FROM ' . CAPTCHA_TABLE . ' WHERE typ="' . $db->escape($type) . '" AND code=CAST("' . $db->escape($captcha) . '" AS BINARY(32))' . ($strict ? ' AND agent=x\'' . $db->escape(md5($_SERVER['HTTP_USER_AGENT'])) . '\'' : ''));
		if($db->affected_rows()){
			$valid[$captcha] = true;
			return true;
		}
		return false;
	}

	public static function cleanup(): void{
		DB_WE::$inst->query('DELETE FROM ' . CAPTCHA_TABLE . ' WHERE valid<NOW()');
	}

	public static function hasTopt(string $totp, int $maxValid = 180): bool{
		if(strlen($totp) > 32){
			$totp = md5($totp, false);
		}
		if(!preg_match('_^(?:[0-9A-Fa-f][0-9A-Fa-f])+$_', $totp)){
			$totp = strlen($totp) > 16 ? md5($totp, false) : bin2hex($totp);
		}
		try{
			if(we_cache_we::isNoCache()){
				if(!fB('SELECT 1 FROM ' . CAPTCHA_TABLE . ' WHERE typ="totp" AND code="' . DB_WE::$inst->escape($totp) . '" AND valid>=NOW()')){
					throw new we_exception_cache('not found');
				}
			} else {
				we_cache_we::load('cap_totp_' . $totp);
			}
			//token found, login must be blocked!
			return true;
		}catch(we_exception_cache $e){
			//token not present, add token, so it can't be used for the next 3 minutes
			self::save($totp, 'totp', $maxValid);
			return false;
		}
	}

	/**
	 * Save the Captcha Code
	 *
	 * @param string $captcha
	 * @return bool
	 */
	public static function save(string $captcha, string $type = 'captcha', int $validity = 1800): bool{
		$db = DB_WE::$inst;
		return
				(we_cache_we::isDBCache() ? false :
				we_cache_we::save('cap_' . $type . '_' . $captcha, 1, $validity)
				) ||
				$db->query('REPLACE INTO ' . CAPTCHA_TABLE . ' SET ' . we_database_base::arraySetter([
							'agent' => empty($_SERVER['HTTP_USER_AGENT']) ? '' : sql_function('x\'' . md5($_SERVER['HTTP_USER_AGENT']) . '\''),
							'typ' => $type,
							'code' => $captcha,
							'valid' => sql_function('NOW()+INTERVAL ' . $validity . ' SECOND'),
		]));
	}

}
