<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 47b6142f8547 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2022-11-01 23:00 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
//@phan-file-suppress PhanUndeclaredClassMethod,PhanUndeclaredMethod,PhanUndeclaredExtendedClass
use Sabre\DAV;

class we_davfs_dbTemplate extends we_davfs_dbFile implements DAV\PartialUpdate\IPatchSupport{

	public const HEADER = '<we:comment id="we-internal">
	Path: __PATH__
	ID:   __ID__
</we:comment>
';
	public const REPLACE = '_<we:comment id="we-internal">([^<]*)</we:comment>\n?_s';

	/**
	 * @return string
	 */
	function get(){
		$templ = new we_contents_template($this->ID);
		if(!$templ->userHasPerms()){
			return 'Permission denied';
		}

		return $this->getHeader() .
				$templ->getElementS('data');
	}

	public function getHeader(): string{
		return strtr(self::HEADER, [//@phan-suppress-current-line PhanParamSuspiciousOrder
			'__PATH__' => $this->path,
			'__ID__' => $this->ID,
		]);
		//FIXME: add additional info like Master-Document
	}

	/**
	 * @return string
	 */
	public function getContentType(){
		return 'text/php';
	}

	/**
	 * @param string|resource $data
	 * @return string|null
	 * @throws DAV\Exception\ConflictingLock
	 * @throws DAV\Exception\PreconditionFailed
	 */
	function put($data){
		$content = is_resource($data) ? (stream_get_contents($data) ?: '') : $data;
		$templ = new we_contents_template($this->ID);
		if($templ->isLockedByUser() || !$templ->lockDocument()){
			t_e('unable to lock template ', $this->path, $this->ID);
			throw new Exception\ConflictingLock();
		}
		$templ->setElementS('data', preg_replace(self::REPLACE, '', $content));
		if(!$templ->we_save() || !$templ->we_publish()){
			t_e('unable to write template ', $this->path, $this->ID);
			we_users_user::unlockDocuments([$this->ID], [$this->Table], we_base_WE::$transaction);
			throw new DAV\Exception\PreconditionFailed('internal error');
		}
		we_users_user::unlockDocuments([$this->ID], [$this->Table], we_base_WE::$transaction);
		return $this->getETag();
	}

	/**
	 *
	 * @param string $name
	 * @return string
	 * @throws DAV\Exception\ConflictingLock
	 * @throws DAV\Exception\PreconditionFailed
	 */
	public function setName($name){
		'@phan-var string $name';
		$templ = new we_contents_template($this->ID);
		if(!$templ->lockDocument()){
			throw new DAV\Exception\ConflictingLock();
		}
		$templ->Filename = preg_replace('|' . $templ->Extension . '$|', '', $name);
		$templ->Path = $templ->getPath();
		if(!$templ->we_save()){
			throw new DAV\Exception\PreconditionFailed('internal error');
		}
		we_users_user::unlockDocuments([$this->ID], [$this->Table], we_base_WE::$transaction);
		return $templ->getText();
	}

	/**
	 * Updates the file based on a range specification.
	 *
	 * The first argument is the data, which is either a readable stream
	 * resource or a string.
	 *
	 * The second argument is the type of update we're doing.
	 * This is either:
	 * * 1. append
	 * * 2. update based on a start byte
	 * * 3. update based on an end byte
	 * ;
	 * The third argument is the start or end byte.
	 *
	 * After a successful put operation, you may choose to return an ETag. The
	 * ETAG must always be surrounded by double-quotes. These quotes must
	 * appear in the actual string you're returning.
	 *
	 * Clients may use the ETag from a PUT request to later on make sure that
	 * when they update the file, the contents haven't changed in the mean
	 * time.
	 *
	 * @param resource|string $data
	 * @param int             $rangeType
	 * @param int             $offset
	 *
	 * @return string|null
	 */
	public function patch($data, $rangeType, $offset = null){
		$templ = new we_contents_template($this->ID);
		if($templ->isLockedByUser() || !$templ->lockDocument()){
			throw new DAV\Exception\ConflictingLock();
		}
		$content = $this->doPatch($templ->getElementS('data'), is_resource($data) ? (stream_get_contents($data) ?: '') : $data, $rangeType, $offset ?: 0);

		$templ->setElementS('data', $content);
		if(!$templ->we_publish()){
			throw new DAV\Exception\PreconditionFailed('internal error');
		}
		we_users_user::unlockDocuments([$this->ID], [$this->Table], we_base_WE::$transaction);
		return $this->getETag();
	}

	/*public static function createFile(int $parentID,$name, $content){
		// We're not allowing dots
		if(in_array($name, ['.', '..']) || !str_ends_with($name, '.tmpl')){
			throw new DAV\Exception\Forbidden('Permission denied to . and ..');
		}

		$templ = new we_contents_template();
		$m = $n = [];
		if(!preg_match(trim(self::REPLACE), $name, $m)){
			t_e($m,preg_replace(self::REPLACE, '', $content));
			throw new DAV\Exception\PreconditionFailed('missing header');
		}
		t_e($m);
		if(preg_match('_ID:\s*(\d+)_', $m[0], $n)){
			if(fB('SELECT 1 FROM ' . TEMPLATES_TABLE . ' WHERE ID=' . ((int) $n[0]) . ' LIMIT 1')){
				t_e('ID ' . $n[0] . ' exists');
				throw new DAV\Exception\PreconditionFailed('ID exists');
			}
			$templ->insertID = $n[0];
		}
		t_e($n);
		throw new DAV\Exception\PreconditionFailed('x');
		$templ->Name = strtr($name, ['.tmpl' => '']);
		$templ->setParentID($parentID);

		$templ->setElementS('data', preg_replace(self::REPLACE, '', $content));
		if(!$templ->we_save() || !$templ->we_publish()){
			t_e('unable to write template ', $this->path, $this->ID);
			we_users_user::unlockDocuments([$this->ID], [$this->Table], we_base_WE::$transaction);
			throw new DAV\Exception\PreconditionFailed('internal error');
		}
		$newPath = $this->path . '/' . $name;
		file_put_contents($newPath, $data);
		clearstatcache(true, $newPath);

		return null;
	}*/

}
