<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 4c49394470f5 $
 * $Author: lukasimhof $
 * $Date: 2022-11-04 01:22 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_dialog_base{
	/** @var we_database_base */
	protected $db;

	/** @var string */
	protected $what = '';

	/** @var array<string,string> */
	protected $argTypes = [];

	/** @var array<string,string|int> */
	protected $args = [];

	/** @var string */
	protected $dialogTitle = '';

	/** @var string[] */
	protected $changeableArgs = [];

	/** @var int */
	protected $pageNr = 1;

	/** @var int */
	protected $numPages = 1;

	/** @var bool */
	protected $onOkJsOnly = false;

	/** @var string */
	protected $bodyId = 'weDialog';

	/** @var bool */
	protected $noInternals = true;

	/** @var string[] */
	protected $we_cmd = [];

	/** @var string */
	protected $cmd = '';

	/** @var string */
	protected $task = '';

	/** @var we_base_jsCmd */
	protected $jsCmd;

	/** @var string */
	protected $tinyDialogName = '';

	/** @var bool */
	protected $isTinyDialog = false;

	/**
	 * Constructor of class.
	 *
	 */
	protected function __construct(bool $noInternals = true){
		$this->noInternals = $noInternals;
		$this->jsCmd = new we_base_jsCmd();
		$this->db = new DB_WE();
	}

	protected function initByHttp(): void{
		$this->what = we_base_request::string('we_what', '');
		$this->we_cmd = we_base_request::stringList('we_cmd');
		$this->cmd = we_base_request::string('we_cmd', '', 0);
		$this->task = we_base_request::string('we_cmd', '', 'task');
		if(($args = we_base_request::stringList('we_dialog_args'))){//assume no tags are allowed
			$this->args = $args;
			foreach($this->args as $key => &$value){
				$value = is_string($value) ? rawurldecode($value) : $value;

				if(!empty($this->argTypes) && isset($this->{$key})){
					if($this->argTypes[$key]){
						$this->{$key} = we_base_request::filterVar($value, $this->argTypes[$key]); //@phan-suppress-current-line PhanDeprecatedFunction
					} else {
						t_e('missing entry in argTypes: ' . $key);
					}
				}
			}
		}
		$this->pageNr = we_base_request::int('we_pageNr', $this->pageNr);
	}

	protected function getHTML(): string{
		if($this->onOkJsOnly){
			$this->what = 'dialog';
		}

		switch($this->what){
			case 'cmd':
				return $this->getCmdHTML();
			default:
				return $this->getHeaderHTML('', $this->getFramesetHTML());
		}
	}

	public static function getDialog(bool $noInternals = true): string{
		$inst = new static($noInternals); //@phan-suppress-current-line PhanTypeInstantiateAbstractStatic

		return $inst->getHTML();
	}

	protected function getCmdHTML(): string{
		$send = [];

		// " quote for correct work within ""
		foreach($this->args as $k => $v){
			$send[$k] = str_replace('"', '\"', (string) $v);
		}

		return $this->cmdFunction($send) . $this->jsCmd->getCmds();
	}

	protected function cmdFunction(array $args): string{
		return '';
	}

	protected function getFramesetHTML(string $extraBody = ''): string{
		return we_html_element::body(['onload' => "weDialogBase_setLoaded();", 'id' => $this->bodyId, 'class' => 'weDialogBody', 'onunload' => "doUnload();"], we_html_element::exIFrame('main', $this->getDialogHTML(), 'position:absolute;top:0;bottom:0;left:0;right:0;overflow: hidden;') .
				we_html_element::iframe('we_cmd_frame', 'about:blank', 'position:absolute;height:0;bottom:0;left:0;right:0;overflow: hidden;') .
				$extraBody
		);
	}

	protected function getBtnNext(): string{
		return we_html_button::create_button(we_html_button::NEXT, "javascript:document.forms['0'].submit();");
	}

	protected function getBtnOk(): string{
		return we_html_button::create_button(we_html_button::OK, "javascript:weDoOk();");
	}

	protected function getBtnCancel(): string{
		return we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();");
	}

	protected function getBtnBack(): string{
		return ($this->pageNr > 1) ? we_html_button::create_button(we_html_button::BACK, "javascript:history.back();") : '';
	}

	protected function getDialogHTML(): string{
		$parts = $this->getDialogPartsHTML();
		if($parts->count()){
			$dialogContent = (new we_html_multiIconBox($parts))->marginLeft()->buttons($this->getDialogButtons())->headline($this->dialogTitle)->get();
		} else {
			$dialogContent = we_html_tools::dialogLayout($this->getDialogContentHTML(), $this->dialogTitle, $this->getDialogButtons());
		}

		return $this->getFormHTML($dialogContent . we_html_element::hidden('we_what', 'cmd') . $this->getHiddenArgs());
	}

	protected function getDialogButtons(): string{
		if($this->pageNr == $this->numPages && $this->onOkJsOnly == false){
			$back = $this->getBtnBack();
			$ok = we_html_button::create_button(we_html_button::OK, we_html_button::WE_FORM . ':we_form');
			$okBut = $back ? $back . $ok : $ok;
		} else if($this->pageNr < $this->numPages){
			$back = $this->getBtnBack();
			$next = $this->getBtnNext();
			$okBut = ($back && $next ) ? $back . $next : ($back ?: $next );
		} else {
			$back = $this->getBtnBack();
			$ok = $this->getBtnOk();
			$okBut = ($back && $ok ) ? $back . $ok : ($back ?: $ok );
		}
		return we_html_button::position_yes_no_cancel($okBut, '', $this->getBtnCancel());
	}

	protected function getFormHTML(string $content): string{
		$hiddens = '';
		if(($cmd = we_base_request::stringList('we_cmd'))){
			unset($cmd['task']);
			foreach($cmd as $k => $v){
				//TODO: why should we loop this commands through?
				$hiddens .= we_html_element::hidden('we_cmd[' . $k . ']', $v);
			}
		}

		//create some empty we_cmds to be filled by JS if needed
		for($i = 0; $i < 4; $i++){
			$hiddens .= isset($_REQUEST['we_cmd'][$i]) ? '' : we_html_element::hidden('we_cmd[' . $i . ']', '');
		}

		$target = (!$this->onOkJsOnly ? ' target="we_cmd_frame"' : '');

		return '<form name="' . we_base_constants::WE_FORM . '" action="' . we_http_util::getScriptName() . '" method="post"' . $target . '>' . $hiddens . $content . '</form>';
	}

	protected function getHiddenArgs(): string{
		$hiddenArgs = '';

		foreach($this->args as $k => $v){
			if(!in_array($k, $this->changeableArgs, true)){
				$hiddenArgs .= we_html_element::hidden('we_dialog_args[' . $k . ']', $v);
			}
		}
		return $hiddenArgs;
	}

	protected function getDialogContentHTML(): string{
		return ''; // overwrite !
	}

	protected function getDialogPartsHTML(): we_html_multiIconBoxContent{
		return new we_html_multiIconBoxContent();
	}

	protected function getHeaderHTML(string $additionals = '', string $body = ''): string{
		return we_html_tools::getHtmlTop($this->dialogTitle, we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_base.js', '', [
					'id' => 'loadVarDialog',
					'data-vars' => setDynamicVar([
						'onEnterKey' => intval($this->pageNr == $this->numPages && $this->onOkJsOnly),
						'pluginName' => $this->tinyDialogName
				])]) .
				($this->isTinyDialog ? self::getTinyMceDialogJS() .
				we_html_element::cssLink(CSS_DIR . 'wysiwyg/tinymce/weDialogCss.css') : ''
				) .
				$this->getJs() .
				$this->getCss() .
				$additionals .
				($this->isTinyDialog ? we_html_element::cssLink(CSS_DIR . 'wysiwyg/tinymce/tinyDialogCss.css') : '') .
				we_html_element::cssLink(CSS_DIR . 'wysiwyg/tinymce/weDialogCss.css') .
				$this->jsCmd->getCmds(), $body);
	}

	protected function getTinyMceDialogJS(): string{
		return we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'weTinyMce_plugin.js') .
			we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'utils/form_utils.js') .
			we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'utils/editable_selects.js');
	}

	protected function getJs(): string{
		return '';
	}

	protected function getCss(): string{
		return '';
	}

	protected function getHttpVar(string $type, string $name, /* mixed */ $alt = '')/* :mixed */{
		return we_base_request::_($type, 'we_dialog_args', $alt, $name);
	}

	protected function getLangField(string $name, string $title, int $width): string{
		return we_html_tools::getLangField('we_dialog_args[' . $name . ']', (string) ($this->args[$name] ?? ''), $title, $width);
	}

	protected function getClassSelect(string $style = 'width: 300px;'): string{
		$clSelect = new we_html_select([
			'name' => 'we_dialog_args[cssclass]',
			'id' => 'we_dialog_args[cssclass]',
			'style' => $style,
			'class' => 'classesSelect',
			'multiple' => true
		]);

		return $clSelect->getHTML();
	}

}
